
local CorralFrame = ZooKeeper.Frame

function CorralFrame:CreateButton(i)

    -- make button name
    local buttonName = "ZooKeeper"
    if isMount == true then
        buttonName = buttonName.."Mount"
    else
        buttonName = buttonName.."Companion"
    end
    buttonName = buttonName..i

    -- make button
    local newButton
    newButton = CreateFrame('CheckButton', buttonName, self, 'ZooKeeperButtonTemplate')
    newButton.frame = self
    newButton.id = i
    newButton.name = buttonName

    newButton:SetChecked(false)
    newButton:SetScript("OnClick", function(self, button) self.frame:PetButtonOnClick(self, button) end)
    newButton:SetScript("OnDragStart", function(button) button.frame:PetButtonOnDragStart(button) end)
    newButton:SetScript("OnReceiveDrag", function(button) button.frame:PetButtonOnReceiveDrag(button) end)
    newButton:SetScript("OnEnter", function(button) button.frame:PetButtonOnEnter(button) end)
    newButton:SetScript("OnLeave", function(button) button.frame:PetButtonOnLeave(button) end)
    newButton:RegisterForDrag("LeftButton")
    newButton:RegisterForClicks("LeftButtonUp", "RightButtonUp")

    if self:IsShown() then
        newButton:Show()
    end

    return newButton
end

function CorralFrame:PositionButton(button, i, currentRow, newRow, lastRow, lastButton)
    local buttonPadding = self.db.profile.buttonPadding

    button.row = currentRow

    -- align the button
    button:ClearAllPoints()
    if i == 1 then
        button:SetPoint("TOPLEFT", self, "TOPLEFT", self.FRAME_PADDING, (self.TOP_PADDING + self.FRAME_PADDING) * -1)
    elseif newRow then
        button:SetPoint("TOPLEFT", lastRow, "BOTTOMLEFT", 0, (buttonPadding * -1))
    else
        button:SetPoint("TOPLEFT", lastButton, "TOPRIGHT", buttonPadding, 0)
    end
end

function CorralFrame:UpdateButton(button, i, filterSet, orderSet)
    if self:IsShown() then
        local creatureID, creatureName, spellID, icon, active;
        local translatedID = self:TranslateID(i, filterSet, orderSet)

        button.petID = translatedID

        if not translatedID or translatedID == nil then
            button:Hide()
            return
        end

        local isShown = button:IsShown()
        if not isShown or isShown == false then
            button:Show()
        end

        creatureID, creatureName, spellID, icon, active = GetCompanionInfo(self.db.char.lastViewed, translatedID)
        button.creatureID = creatureID
        button.spellID = spellID
        button.active = active

        if creatureID then
            button:SetNormalTexture(icon)
            button:Enable()
        else
            button:Disable()
        end

        if button.petID == self.selectedID then
            button:SetChecked(true)
        else
            button:SetChecked(false)
        end

        if ( active ) then
            _G[button.name.."ActiveTexture"]:Show();
        else
            _G[button.name.."ActiveTexture"]:Hide();
        end
    end
end

function CorralFrame:TranslateID(id, filterSet, orderSet)
    local returnID

    if filterSet and #filterSet > 0 then
        if id > #filterSet then
            returnID = nil
        else
            returnID = filterSet[id]
        end
    else
        returnID = orderSet[id]
    end

    return returnID
end

--[[
    Button Events
--]]
function CorralFrame:PetButtonOnClick(petButton, mouseButton)
    if IsModifiedClick() then
        self:PetButtonOnModifiedClick(petButton, mouseButton)
        return
    end

    local mode = self:GetCurrentMode()
    local selectedID = petButton.petID
    local creatureID, creatureName, spellID, icon, active = GetCompanionInfo(mode, selectedID);
    if ( mouseButton ~= "LeftButton" or (self.selectedID and ( selectedID == self.selectedID ) ) ) then
        if ( active ) then
            DismissCompanion(mode);
            PlaySound("igMainMenuOptionCheckBoxOn");
        else
            CallCompanion(mode, selectedID);
            if ( self:GetCurrentMode() == TYPE_COMPANIONS ) then
                self.db.char.lastCompanion = selectedID
            end
            PlaySound("igMainMenuOptionCheckBoxOff");
        end
    end
    self.selectedID = selectedID
    self:UpdatePets()
end

function CorralFrame:PetButtonOnDragStart(button)
    local offset;

    self.draggedID = button.id
    PickupCompanion( self:GetCurrentMode(), button.petID);
end

function CorralFrame:PetButtonOnReceiveDrag(button)
    local buttonSet = self:GetButtonSet()
    local orderSet = self:GetOrderSet()
    local swapID

    if self.draggedID and self.draggedID ~= nil then
        swapID = orderSet[self.draggedID]
        orderSet[self.draggedID] = orderSet[button.id]
        orderSet[button.id] = swapID
        self:UpdatePets()
    end

    ClearCursor()
    self.draggedID = nil
end

function CorralFrame:PetButtonOnModifiedClick(petButton, mouseButton)
    local id = petButton.spellID;
    if ( IsModifiedClick("CHATLINK") ) then
        if ( MacroFrame and MacroFrame:IsShown() ) then
            local spellName = GetSpellInfo(id);
            ChatEdit_InsertLink(spellName);
        else
            local spellLink = GetSpellLink(id)
            ChatEdit_InsertLink(spellLink);
        end
    elseif ( IsModifiedClick("PICKUPACTION") ) then
        self:PetButtonOnDrag(petButton);
    end
end

function CorralFrame:PetButtonOnEnter(button)
    if ( GetCVar("UberTooltips") == "1" ) then
        GameTooltip_SetDefaultAnchor(GameTooltip, button);
    else
        GameTooltip:SetOwner(self, "ANCHOR_LEFT");
    end

    if ( GameTooltip:SetHyperlink("spell:"..button.spellID) ) then
        self.UpdateTooltip = CompanionButton_OnEnter
    else
        self.UpdateTooltip = nil;
    end

    GameTooltip:Show()
end

function CorralFrame:PetButtonOnLeave(button)
    GameTooltip:Hide()
end
